@@ -1,6 +1,6 @@
-use std::convert::TryFrom;
-use std::fmt;
-use std::str::FromStr;
+use core::convert::TryFrom;
+use core::fmt;
+use core::str::FromStr;
 
 #[cfg(feature = "rust-rgb")]
 use rgb::{RGB, RGBA};
@@ -8,12 +8,18 @@ use rgb::{RGB, RGBA};
 #[cfg(feature = "serde")]
 use serde::{de::Visitor, Deserialize, Deserializer, Serialize, Serializer};
 
-#[cfg(feature = "lab")]
 use crate::lab::{lab_to_linear_rgb, linear_rgb_to_lab};
 
 use crate::utils::*;
 use crate::{parse, ParseColorError};
 
+use alloc::format;
+use alloc::string::String;
+use alloc::string::ToString;
+
+#[cfg(not(feature = "std"))]
+use num_traits::float::Float;
+
 #[cfg(feature = "named-colors")]
 use crate::NAMED_COLORS;
 
@@ -47,7 +53,7 @@ impl Color {
     /// * `g`: Green value [0..255]
     /// * `b`: Blue value [0..255]
     /// * `a`: Alpha value [0..255]
-    pub fn from_rgba8(r: u8, g: u8, b: u8, a: u8) -> Self {
+    pub const fn from_rgba8(r: u8, g: u8, b: u8, a: u8) -> Self {
         Self {
             r: r as f32 / 255.0,
             g: g as f32 / 255.0,
@@ -93,7 +99,7 @@ impl Color {
     /// * `s`: Saturation [0..1]
     /// * `v`: Value [0..1]
     /// * `a`: Alpha [0..1]
-    pub fn from_hsva(h: f32, s: f32, v: f32, a: f32) -> Self {
+    pub const fn from_hsva(h: f32, s: f32, v: f32, a: f32) -> Self {
         let [r, g, b] = hsv_to_rgb(normalize_angle(h), s.clamp(0.0, 1.0), v.clamp(0.0, 1.0));
         Self::new(r, g, b, a)
     }
@@ -104,7 +110,7 @@ impl Color {
     /// * `s`: Saturation [0..1]
     /// * `l`: Lightness [0..1]
     /// * `a`: Alpha [0..1]
-    pub fn from_hsla(h: f32, s: f32, l: f32, a: f32) -> Self {
+    pub const fn from_hsla(h: f32, s: f32, l: f32, a: f32) -> Self {
         let [r, g, b] = hsl_to_rgb(normalize_angle(h), s.clamp(0.0, 1.0), l.clamp(0.0, 1.0));
         Self::new(r, g, b, a)
     }
@@ -115,7 +121,7 @@ impl Color {
     /// * `w`: Whiteness [0..1]
     /// * `b`: Blackness [0..1]
     /// * `a`: Alpha [0..1]
-    pub fn from_hwba(h: f32, w: f32, b: f32, a: f32) -> Self {
+    pub const fn from_hwba(h: f32, w: f32, b: f32, a: f32) -> Self {
         let [r, g, b] = hwb_to_rgb(normalize_angle(h), w.clamp(0.0, 1.0), b.clamp(0.0, 1.0));
         Self::new(r, g, b, a)
     }
@@ -141,7 +147,6 @@ impl Color {
         Self::from_oklaba(l, c * h.cos(), c * h.sin(), alpha)
     }
 
-    #[cfg(feature = "lab")]
     /// Arguments:
     ///
     /// * `l`: Lightness
@@ -153,7 +158,6 @@ impl Color {
         Self::from_linear_rgba(r, g, b, alpha)
     }
 
-    #[cfg(feature = "lab")]
     /// Arguments:
     ///
     /// * `l`: Lightness
@@ -169,7 +173,7 @@ impl Color {
     /// # Examples
     /// ```
     /// use csscolorparser::Color;
-    /// # use std::error::Error;
+    /// # use core::error::Error;
     /// # fn main() -> Result<(), Box<dyn Error>> {
     ///
     /// let c = Color::from_html("rgb(255,0,0)")?;
@@ -186,7 +190,8 @@ impl Color {
     }
 
     /// Restricts R, G, B, A values to the range [0..1].
-    pub fn clamp(&self) -> Self {
+    #[must_use = "method returns a new Color and does not mutate the original Color"]
+    pub const fn clamp(&self) -> Self {
         Self {
             r: self.r.clamp(0.0, 1.0),
             g: self.g.clamp(0.0, 1.0),
@@ -209,9 +214,9 @@ impl Color {
     #[cfg(feature = "named-colors")]
     pub fn name(&self) -> Option<&'static str> {
         let rgb = &self.to_rgba8()[0..3];
-        for (&k, &v) in NAMED_COLORS.entries() {
+        for (&k, v) in NAMED_COLORS.entries() {
             if v == rgb {
-                return Some(k);
+                return Some(k.as_str());
             }
         }
         None
@@ -220,7 +225,7 @@ impl Color {
     /// Returns: `[r, g, b, a]`
     ///
     /// * Red, green, blue and alpha in the range [0..1]
-    pub fn to_array(&self) -> [f32; 4] {
+    pub const fn to_array(&self) -> [f32; 4] {
         [
             self.r.clamp(0.0, 1.0),
             self.g.clamp(0.0, 1.0),
@@ -232,7 +237,7 @@ impl Color {
     /// Returns: `[r, g, b, a]`
     ///
     /// * Red, green, blue and alpha in the range [0..255]
-    pub fn to_rgba8(&self) -> [u8; 4] {
+    pub const fn to_rgba8(&self) -> [u8; 4] {
         [
             (self.r * 255.0 + 0.5) as u8,
             (self.g * 255.0 + 0.5) as u8,
@@ -244,7 +249,7 @@ impl Color {
     /// Returns: `[r, g, b, a]`
     ///
     /// * Red, green, blue and alpha in the range [0..65535]
-    pub fn to_rgba16(&self) -> [u16; 4] {
+    pub const fn to_rgba16(&self) -> [u16; 4] {
         [
             (self.r * 65535.0 + 0.5) as u16,
             (self.g * 65535.0 + 0.5) as u16,
@@ -259,7 +264,7 @@ impl Color {
     /// * `s`: Saturation [0..1]
     /// * `v`: Value [0..1]
     /// * `a`: Alpha [0..1]
-    pub fn to_hsva(&self) -> [f32; 4] {
+    pub const fn to_hsva(&self) -> [f32; 4] {
         let [h, s, v] = rgb_to_hsv(
             self.r.clamp(0.0, 1.0),
             self.g.clamp(0.0, 1.0),
@@ -279,7 +284,7 @@ impl Color {
     /// * `s`: Saturation [0..1]
     /// * `l`: Lightness [0..1]
     /// * `a`: Alpha [0..1]
-    pub fn to_hsla(&self) -> [f32; 4] {
+    pub const fn to_hsla(&self) -> [f32; 4] {
         let [h, s, l] = rgb_to_hsl(
             self.r.clamp(0.0, 1.0),
             self.g.clamp(0.0, 1.0),
@@ -299,7 +304,7 @@ impl Color {
     /// * `w`: Whiteness [0..1]
     /// * `b`: Blackness [0..1]
     /// * `a`: Alpha [0..1]
-    pub fn to_hwba(&self) -> [f32; 4] {
+    pub const fn to_hwba(&self) -> [f32; 4] {
         let [h, w, b] = rgb_to_hwb(
             self.r.clamp(0.0, 1.0),
             self.g.clamp(0.0, 1.0),
@@ -359,7 +364,6 @@ impl Color {
         [l, c, h, alpha]
     }
 
-    #[cfg(feature = "lab")]
     /// Returns: `[l, a, b, alpha]`
     pub fn to_laba(&self) -> [f32; 4] {
         let [r, g, b, alpha] = self.to_linear_rgba();
@@ -367,7 +371,6 @@ impl Color {
         [l, a, b, alpha.clamp(0.0, 1.0)]
     }
 
-    #[cfg(feature = "lab")]
     /// Returns: `[l, c, h, alpha]`
     pub fn to_lcha(&self) -> [f32; 4] {
         let [l, a, b, alpha] = self.to_laba();
@@ -436,7 +439,6 @@ impl Color {
         format!("oklch({l} {c} {h}{})", fmt_alpha(alpha))
     }
 
-    #[cfg(feature = "lab")]
     /// Get CSS `lab()` color representation
     pub fn to_css_lab(&self) -> String {
         let [l, a, b, alpha] = self.to_laba();
@@ -446,10 +448,9 @@ impl Color {
         format!("lab({l} {a} {b}{})", fmt_alpha(alpha))
     }
 
-    #[cfg(feature = "lab")]
     /// Get CSS `lch()` color representation
     pub fn to_css_lch(&self) -> String {
-        use std::f32::consts::PI;
+        use core::f32::consts::PI;
 
         fn to_degrees(t: f32) -> f32 {
             if t > 0.0 {
@@ -467,7 +468,7 @@ impl Color {
     }
 
     /// Blend this color with the other one, in the RGB color-space. `t` in the range [0..1].
-    pub fn interpolate_rgb(&self, other: &Color, t: f32) -> Self {
+    pub const fn interpolate_rgb(&self, other: &Color, t: f32) -> Self {
         Self {
             r: self.r + t * (other.r - self.r),
             g: self.g + t * (other.g - self.g),
@@ -489,7 +490,7 @@ impl Color {
     }
 
     /// Blend this color with the other one, in the HSV color-space. `t` in the range [0..1].
-    pub fn interpolate_hsv(&self, other: &Color, t: f32) -> Self {
+    pub const fn interpolate_hsv(&self, other: &Color, t: f32) -> Self {
         let [h1, s1, v1, a1] = self.to_hsva();
         let [h2, s2, v2, a2] = other.to_hsva();
         Self::from_hsva(
@@ -512,7 +513,6 @@ impl Color {
         )
     }
 
-    #[cfg(feature = "lab")]
     /// Blend this color with the other one, in the Lab color-space. `t` in the range [0..1].
     pub fn interpolate_lab(&self, other: &Color, t: f32) -> Self {
         let [l1, a1, b1, alpha1] = self.to_laba();
@@ -525,7 +525,6 @@ impl Color {
         )
     }
 
-    #[cfg(feature = "lab")]
     /// Blend this color with the other one, in the LCH color-space. `t` in the range [0..1].
     pub fn interpolate_lch(&self, other: &Color, t: f32) -> Self {
         let [l1, c1, h1, alpha1] = self.to_lcha();
@@ -572,6 +571,14 @@ impl TryFrom<&str> for Color {
     }
 }
 
+impl TryFrom<String> for Color {
+    type Error = ParseColorError;
+
+    fn try_from(s: String) -> Result<Self, Self::Error> {
+        parse(s.as_ref())
+    }
+}
+
 impl From<(f32, f32, f32, f32)> for Color {
     fn from((r, g, b, a): (f32, f32, f32, f32)) -> Self {
         Self { r, g, b, a }
@@ -694,7 +701,7 @@ impl Visitor<'_> for ColorVisitor {
 }
 
 fn fmt_float(t: f32, precision: usize) -> String {
-    let s = format!("{:.1$}", t, precision);
+    let s = format!("{t:.precision$}");
     s.trim_end_matches('0').trim_end_matches('.').to_string()
 }
 
